#version 300 es
precision highp float;
uniform mat3 normal_matrix;
uniform vec4 u_ambient;
uniform vec4 u_specular;
uniform vec4 u_diffuse;
uniform float u_specularpower;

uniform sampler2D t_texture0;
uniform sampler2D t_texture1;

in vec2 v_texcoord0;
in vec3 v_normal;
in vec3 v_viewDir;
in vec3 v_lightDir;
in vec4 v_color;

out vec4 outColor;

void main( void )
{
    vec3 tangent;
    vec3 binormal;

    vec3 c1 = cross(v_normal, vec3(0, 0, 1));
    vec3 c2 = cross(v_normal, vec3(0, 1, 0));

    if (length(c1) > length(c2))
        tangent = c1;
    else
        tangent = c2;

    tangent = normalize(tangent);
    binormal = cross(v_normal, tangent);
    binormal = normalize(binormal);

    mat3 tangMat = mat3(tangent, binormal, v_normal);

    vec3 fvNormal = texture(t_texture1, v_texcoord0).rgb;
    fvNormal = fvNormal * 2.0 - 1.0;
    fvNormal = tangMat * fvNormal;

    float fNDotL           = dot( fvNormal, v_lightDir ); 
    vec3  fvReflection     = normalize( ( ( 2.0 * fvNormal ) * fNDotL ) - v_lightDir );
   
    vec4  fvBaseColor      = v_color * texture( t_texture0, v_texcoord0 );
   
    vec4  fvTotalDiffuse   = vec4((u_diffuse * fNDotL).xyz, 1.0) * fvBaseColor;
    vec4  fvTotalSpecular  = u_specular * ( pow( max( 0.0, dot( fvReflection, v_viewDir ) ), u_specularpower ) );

    outColor = vec4(( (u_ambient + fvTotalDiffuse + fvTotalSpecular) * fvBaseColor ).xyz, v_color.w);
}